package org.springframework.batch;

import javax.sql.DataSource;

import org.springframework.batch.core.Job;
import org.springframework.batch.core.JobExecution;
import org.springframework.batch.core.JobInstance;
import org.springframework.batch.core.JobParameter;
import org.springframework.batch.core.JobParameters;
import org.springframework.batch.core.JobParametersBuilder;
import org.springframework.batch.core.Step;
import org.springframework.batch.core.configuration.JobLocator;
import org.springframework.batch.core.configuration.JobRegistry;
import org.springframework.batch.core.configuration.annotation.EnableBatchProcessing;
import org.springframework.batch.core.configuration.support.JobRegistryBeanPostProcessor;
import org.springframework.batch.core.job.builder.JobBuilder;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.batch.core.repository.JobRepository;
import org.springframework.batch.core.step.builder.StepBuilder;
import org.springframework.batch.repeat.RepeatStatus;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabaseBuilder;
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabaseType;
import org.springframework.jdbc.support.JdbcTransactionManager;

import java.util.List;

@Configuration
@EnableBatchProcessing
public class MyBatchJobConfiguration {

	/*
	 * Dummy Job to be deleted
	 */
	private static final String DUMMY_JOB = "dummyJob";

	@Bean(name = "Step" + DUMMY_JOB)
	public Step dummyStep(JobRepository jobRepository, JdbcTransactionManager transactionManager) {
		return new StepBuilder("step", jobRepository)
			.tasklet((contribution, chunkContext) -> {
				System.out.println("hello world");
				return RepeatStatus.FINISHED;
			}, transactionManager)
			.build();
	}

	@Bean(name = "Job" + DUMMY_JOB)
	public Job dummyJob(JobRepository jobRepository, @Qualifier("Step" + DUMMY_JOB) Step step) {
		return new JobBuilder(DUMMY_JOB, jobRepository)
			.start(step)
			.build();
	}

	/*
	 * Job that delete other jobsInstances
	 */
	private static final String DELETE_JOB = "deleteJob";

	@Bean(name = "Step" + DELETE_JOB)
	public Step deleteStep(JobRepository jobRepository, JdbcTransactionManager transactionManager) {
		return new StepBuilder("step", jobRepository)
			.tasklet((contribution, chunkContext) -> {
				List<JobInstance> jobInstancesByName = jobRepository.findJobInstancesByName(DUMMY_JOB, 0, 5);
				jobInstancesByName.forEach(jobRepository::deleteJobInstance);
				/*
				 * This code throw Caused by: org.h2.jdbc.JdbcSQLIntegrityConstraintViolationException
				 * Referential integrity constraint violation
				 * SQL statement: DELETE FROM BATCH_JOB_EXECUTION WHERE JOB_EXECUTION_ID
				 */
				return RepeatStatus.FINISHED;
			}, transactionManager)
			.build();
	}

	@Bean(name = "Job" + DELETE_JOB)
	public Job deleteJob(JobRepository jobRepository, @Qualifier("Step" + DELETE_JOB) Step step) {
		return new JobBuilder(DELETE_JOB, jobRepository)
			.start(step)
			.build();
	}
	/*
	 * Infrastructure beans configuration
	 */

	@Bean
	public DataSource dataSource() {
		return new EmbeddedDatabaseBuilder()
			.setType(EmbeddedDatabaseType.H2)
			.addScript("/org/springframework/batch/core/schema-h2.sql")
			.build();
	}

	@Bean
	public JdbcTransactionManager transactionManager(DataSource dataSource) {
		return new JdbcTransactionManager(dataSource);
	}

	@Bean
	public JobRegistryBeanPostProcessor jobRegistryBeanPostProcessor(JobRegistry jobRegistry) {
		JobRegistryBeanPostProcessor jobRegistryBeanPostProcessor = new JobRegistryBeanPostProcessor();
		jobRegistryBeanPostProcessor.setJobRegistry(jobRegistry);
		return jobRegistryBeanPostProcessor;
	}

	/*
	 * Main method to run the application and exhibit the issue
	 */
	public static void main(String[] args) throws Exception {
		ApplicationContext context = new AnnotationConfigApplicationContext(MyBatchJobConfiguration.class);
		JobLauncher jobLauncher = context.getBean(JobLauncher.class);
		JobLocator jobLocator = context.getBean(JobLocator.class);


		/*
		 * Launch 3 dummy jobs
		 */
		Job jobdummyJob = jobLocator.getJob("dummyJob");

		JobParametersBuilder jobParametersBuilder = new JobParametersBuilder();
		jobParametersBuilder.addString("id", "1");
		JobExecution jobExecution = jobLauncher.run(jobdummyJob, jobParametersBuilder.toJobParameters());
		System.out.println(jobExecution.getExitStatus().getExitCode());

		JobParametersBuilder jobParametersBuilder2 = new JobParametersBuilder();
		jobParametersBuilder2.addString("id", "2");
		JobExecution jobExecution2 = jobLauncher.run(jobdummyJob, jobParametersBuilder2.toJobParameters());
		System.out.println(jobExecution2.getExitStatus().getExitCode());

		JobParametersBuilder jobParametersBuilder3 = new JobParametersBuilder();
		jobParametersBuilder3.addString("id", "3");
		JobExecution jobExecution3 = jobLauncher.run(jobdummyJob, jobParametersBuilder3.toJobParameters());
		System.out.println(jobExecution3.getExitStatus().getExitCode());

		/*
		 * Launch the deleting job
		 */
		Job deleteJob = jobLocator.getJob("deleteJob");
		JobParametersBuilder jobParametersBuilderDelete = new JobParametersBuilder();
		jobParametersBuilder.addString("id", "delete");
		JobExecution deleteJobExecution = jobLauncher.run(deleteJob, jobParametersBuilderDelete.toJobParameters());
		System.out.println(deleteJobExecution.getExitStatus().getExitCode());
	}
}